/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

/**
 * Common ancestor of all "raw" trace messages, which are being processed on the level of {@link
 * MessageStream message streams}. {@link AspectAdapter Aspect adapters} converts them into the
 * respective {@link AspectMessage Aspect messages}.
 *
 * @author Sergey Groshev
 */
public abstract class AbstractTraceMessage implements java.io.Serializable {
  public enum Kind {
    /** Class {@link TraceMessage}.*/
    TRACE_MESSAGE,
    /** Class {@link TagMessage}.*/
    TAG_MESSAGE,
    /** Class {@link DebugMessage}.*/
    DEBUG_MESSAGE
  }

  /**
   * Message type determiner. Trace message handlers ({@link Tracer.Channel trace channels}, {@link
   * MessageStreamFilter trace filters}, realizations {@link TraceProvider trace providers} и {@link
   * MessageStream message streams} etc.) can use this method instead of checking for {@code
   * instanceof}.
   */
  public abstract Kind getKind();

  private final String aspect;
  private final String channel;

  /**
   * Constructor.
   *
   * @param aspect Aspect.
   * @param channel Channel identifier. Can be {@code null} - in this case we take the channel
   *     identifier from an actual tracer.
   */
  protected AbstractTraceMessage(String aspect, String channel) {
    this.aspect = aspect;
    this.channel = channel == null ? Tracer.getChannel() : channel;
  }

  /** Gets message aspect. */
  public final String getAspect() {
    return aspect;
  }

  /** Gets channel identifier. */
  public final String getChannel() {
    return channel;
  }
}
