/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

import com.unitesk.aspectrace.tracer.TraceToProcessor;

/**
 * An interface for direct information transmission which is hard to convert to a tree {@link
 * TraceNode of nodes}. If data processing comes with generation via {@link TraceToProcessor direct
 * processing} then data is transmitted directly from a tracer to an adapter. If it is necessary to
 * convert data for an external transmission then we call {@link #toNode() delayed conversion}.
 * Delayed conversion may cast an exception - in this case system will be correctly working only in
 * case of direct trace processing without any intermediate external displays.
 *
 * @author Sergey Groshev
 */
public interface LazyNode {

  /**
   * Delayed conversion.
   *
   * @return Node tree.
   * @throws UnsupportedOperationException Conversion is not supported.
   */
  public TraceNode toNode() throws UnsupportedOperationException;
}
