/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

/**
 * {@link MessageStream message stream} Listener.
 *
 * @author Sergey Groshev
 */
public interface MessageStreamListener {

  /**
   * Stream start. If the stream has been created earlier or the listener was connected to the
   * stream after the start then the stream start message is not transmitted.
   *
   * @throws Throwable An error has occurred. {@link MessageStream} ignores it anyway.
   */
  public void handleStreamStart() throws Throwable;

  /**
   * Message arrival.
   *
   * @param msg Message.
   * @throws Throwable An error has occurred. {@link MessageStream} ignores it anyway.
   */
  public void handleStreamMessage(AbstractTraceMessage msg) throws Throwable;

  /**
   * Stream end or disconnection.
   *
   * @param normal {@code true} if the end is normal else {@code false}.
   * @throws Throwable An error has occurred. {@link MessageStream} ignores it anyway.
   */
  public void handleStreamEnd(boolean normal) throws Throwable;
}
