/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.common;

import com.unitesk.aspectrace.AspectAdapter;
import com.unitesk.aspectrace.TagMessage;

/**
 * The empty handler of empty aspect messages. Defines all necessary methods with empty realizations.
 * The method {@link AbstractCommonListener#processTagChange(TagMessage)} must not be called thus his
 * realization throws an exception.
 */
public abstract class AbstractCommonListener implements CommonListener {
  @Override
  public void attachedToAdapter(final String aspect, final AspectAdapter adapter)
      throws IllegalStateException {}

  @Override
  public void processingStarted(final String traceDesc) {}

  @Override
  public void processGlobalTagChange(final TagMessage msg) {}

  @Override
  public void processChannelMessage(final ChannelMessage msg) {}

  @Override
  public void processingFinished(final String traceDesc, final boolean normally) {}

  @Override
  public void processTagChange(final TagMessage msg) {
    // Should never be called.
    throw new UnsupportedOperationException("Tag for default aspect");
  }
}
