/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.error;

/**
 * Error message.
 *
 * @author Sergey Groshev
 */
public class FailureMessage extends com.unitesk.aspectrace.AspectMessage {

  String originAspect;
  String shortDesc;
  String fullDesc;
  Object data;
  StackTraceElement[] stack; // Forced non-null by adapter

  FailureMessage(ErrorAdapter adapter) {
    super(adapter);
  }

  /** Gets aspect which created error. */
  public String getOriginAspect() {
    return originAspect;
  }

  /** Gets short error description. */
  public String getShortDesc() {
    return shortDesc;
  }

  /** Gets full error description. */
  public String getFullDesc() {
    return fullDesc;
  }

  /** Gets attached data. */
  public Object getData() {
    return data;
  }

  /**
   * Gets call stack describing error source.
   *
   * @return Array of stack describers. Returned array is not bound to message therefore can be changed as needed.
   */
  public StackTraceElement[] getStack() {
    return stack.clone();
  }

  /** Gets call stack size. */
  public int getStackSize() {
    return stack.length;
  }

  /** Gets call stack element by its index. */
  public StackTraceElement getStackElement(int index) throws IndexOutOfBoundsException {
    return stack[index];
  }
}
