/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.html;

/**
 * Reference from one report HTML-page to another.
 * Uses target page method {@link HtmlReportPage#generateRef(HtmlReportPage, String)}
 * call for HTML-code reference generation by default, however subclasses can change this behaviour.
 *
 * @author Sergey Groshev
 */
public class PageRef {

  protected HtmlReportPage from;
  protected HtmlReportPage to;
  protected String mark;

  public PageRef(HtmlReportPage from, HtmlReportPage to) {
    this(from, to, null);
  }

  public PageRef(HtmlReportPage from, HtmlReportPage to, String mark) {
    this.from = from;
    this.to = to;
    this.mark = mark;
  }

  public HtmlReportPage getFrom() {
    return from;
  }

  public HtmlReportPage getTo() {
    return to;
  }

  public String getMark() {
    return mark;
  }

  public String generateRef() {
    return to.generateRef(from, mark);
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((from == null) ? 0 : from.hashCode());
    result = prime * result + ((to == null) ? 0 : to.hashCode());
    result = prime * result + ((mark == null) ? 0 : mark.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (obj == this) {
      return true;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }

    PageRef pr = (PageRef) obj;
    return pr.from == from && pr.to == to
            && (mark == null ? pr.mark == null : mark.equals(pr.mark));
  }
}
