/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.impl;

import com.unitesk.aspectrace.AspectAdapter;
import com.unitesk.aspectrace.Parameter;
import com.unitesk.aspectrace.TraceMessage;
import com.unitesk.aspectrace.TraceNode;

import java.util.List;

/**
 * Adapter for aspect "impl".
 *
 * @author Sergey Groshev
 */
public class ImplAdapter extends AspectAdapter {

  @Override
  public String getAspect() {
    return "impl";
  }

  @Override
  public Class<ImplListener> getListenerClass() {
    return ImplListener.class;
  }

  @Override
  protected void processMessage(TraceMessage msg) throws IllegalArgumentException {
    super.processMessage(msg);

    String msgName = msg.getName();
    if (msgName.equals("start")) {
      MethodStartMessage msm = new MethodStartMessage(this);
      msm.signature = msg.getAttr("name");
      String id = msg.getAttr("id");
      if (id != null) {
        try {
          msm.id = Integer.parseInt(id);
        } catch (NumberFormatException nfe) {
          throw new IllegalArgumentException("Invalid start id:" + id, nfe);
        }
      }
      for (TraceNode n : msg.getNode().getChildren()) {
        msm.parameters.add(Parameter.fromNode(n));
      }
      for (ImplListener lst : getListeners()) {
        lst.processMethodStart(msm);
      }
    } else if (msgName.equals("end")) {
      MethodEndMessage mem = new MethodEndMessage(this);
      mem.signature = msg.getAttr("name");
      String id = msg.getAttr("id");
      if (id != null) {
        try {
          mem.id = Integer.parseInt(id);
        } catch (NumberFormatException nfe) {
          throw new IllegalArgumentException("Invalid end id:" + id, nfe);
        }
      }
      for (TraceNode ch : msg.getChildren()) {
        if (ch.isNode() && ch.getName().equals("return")) {
          mem.retVal = ch.getChild(0).getText();
        } else if (ch.isNode() && ch.getName().equals("exception")) {
          mem.exception = ch.getChild(0).getText();
        } else {
          throw new IllegalArgumentException("Invalid structure of method end message");
        }
      }
      for (ImplListener lst : getListeners()) {
        lst.processMethodEnd(mem);
      }
    } else {
      throw new IllegalArgumentException("Unsupported impl node: " + msgName);
    }
  }

  @Override
  protected List<ImplListener> getListeners() {
    return (List<ImplListener>) super.getListeners();
  }
}
