/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.provider;

import com.unitesk.aspectrace.tracer.XmlFormatter;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Провайдер трассы: трасса получается при разборе XML-файла, отформатированного с помощью {@link
 * XmlFormatter}.
 *
 * @author Sergey Groshev
 */
public class XmlFileParser extends XmlStreamParser {

  private File traceFile;

  public XmlFileParser() {}

  public XmlFileParser(String trace) {
    setTrace(trace);
  }

  public XmlFileParser(File trace) {
    setTrace(trace);
  }

  public void setTrace(String trace) {
    setTrace(new File(trace));
  }

  public void setTrace(File trace) {
    if (traceFile != null) {
      throw new IllegalStateException("trace already set");
    }
    traceFile = trace;
  }

  @Override
  protected InputStream getInputStream() throws IOException {
    return new BufferedInputStream(new FileInputStream(traceFile));
  }

  @Override
  protected String getSysId() {
    return traceFile.toString();
  }

  @Override
  protected String getTraceDesc() {
    return getSysId();
  }
}
