/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.sampleApp;

import com.unitesk.aspectrace.fsm.FsmEndMessage;
import com.unitesk.aspectrace.fsm.FsmStateMessage;
import com.unitesk.aspectrace.fsm.TransitionMessage;
import com.unitesk.aspectrace.html.ReportGenerator;
import com.unitesk.aspectrace.impl.ImplListener;
import com.unitesk.aspectrace.impl.MethodEndMessage;
import com.unitesk.aspectrace.impl.MethodStartMessage;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Arrays;

public class FsmAndImpl extends FsmGraphProcessor implements ImplListener {

  StringWriter implBuff = new StringWriter();
  PrintWriter implW = new PrintWriter(implBuff);

  public FsmAndImpl(ReportGenerator rg, ErrorCollector ec) {
    super(rg, ec);
  }

  @Override
  public void processMethodStart(MethodStartMessage msm) {
    if (curPage.get(msm.getChannel()) == null) {
      return;
    }

    implW.print(
        "<i>Impl call: " + msm.getSignature() + ": " + Arrays.toString(msm.getParameters()));
    implW.println("<br/></i>");
  }

  @Override
  public void processMethodEnd(MethodEndMessage mem) {
    if (curPage.get(mem.getChannel()) == null) {
      return;
    }

    implW.println("<i>");
    if (mem.getException() != null) {
      implW.print("... exception: " + mem.getException());
    } else if (mem.getRetVal() != null) {
      implW.print("... return: " + mem.getRetVal());
    } else {
      implW.print("... returned normally");
    }
    implW.println("<br/></i>");
  }

  private void flushBuffer(FsmPage page) {
    page.getPw().print(implBuff.toString());
    implBuff.getBuffer().setLength(0);
  }

  @Override
  public void processStateMessage(FsmStateMessage msg) {
    flushBuffer(curPage.get(msg.getChannel()));
    super.processStateMessage(msg);
  }

  // Designed to work with logs of MethodInvoker and DFSExplorer from MBTest
  @Override
  public void processTransitionMessage(TransitionMessage msg) {
    FsmPage page = curPage.get(msg.getChannel());
    if (msg.isStart()) {
      flushBuffer(page);
      super.processTransitionMessage(msg);
    } else {
      super.processTransitionMessage(msg); // Write out transition header
      flushBuffer(page);
    }
  }

  @Override
  public void processFsmEnd(FsmEndMessage msg) {
    flushBuffer(curPage.get(msg.getChannel()));
    super.processFsmEnd(msg);
  }
}
