/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.sampleApp;

import com.unitesk.aspectrace.common.AbstractCommonListener;
import com.unitesk.aspectrace.html.HtmlReportPage;
import com.unitesk.aspectrace.ref.ContextProvider;

import java.io.PrintWriter;

public class TraceTracker extends AbstractCommonListener
    implements ContextProvider<HtmlReportPage> {
  String traceDesc = null;

  @Override
  public void processingStarted(String traceDesc) {
    this.traceDesc = traceDesc;
    System.out.println("Trace processing started: " + traceDesc);
  }

  @Override
  public void processingFinished(String traceDesc, boolean normally) {
    System.out.println("Trace processing finished: " + traceDesc);
    traceDesc = null;
  }

  @Override
  public HtmlReportPage[] getRelated(final String channel, HtmlReportPage target) {
    if (traceDesc == null) {
      return null;
    }
    final String desc = traceDesc;
    return new HtmlReportPage[] {
      new HtmlReportPage() {
        @Override
        public void generatePageBody(PrintWriter pw) {
          pw.println("<table class=\"section\" width=\"100%\">");
          pw.println("<tr><th>Trace");
          pw.println("<tr><td>" + encodeSimple(desc));
          pw.println("<tr><th>Channel");
          pw.println("<tr><td>" + encodeSimple(channel));
          pw.println("</table>");
        }
      }
    };
  }
}
