/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.templates;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Calendar;

/**
 * Document context for JavaTESK and Ch@se reports
 *
 * @version $Revision: 1.49 $
 * @author <a href="mailto:sergg@ispras.ru">Sergey Gomanyuk</a>
 */
public class DocumentContext {

  public static final String TEMPLATE_PACKAGE = "com/unitesk/aspectrace/templates/vmTemplates/";
  public static final String GIF_PACKAGE = "com/unitesk/aspectrace/templates/templates/";

  public static final String OVERVIEW = "overview";
  public static final String OVERVIEW_B = "overview_b";
  public static final String OVERVIEW_M = "overview_m";
  public static final String OVERVIEW_P = "overview_p";
  public static final String OVERVIEW_D = "overview_d";
  public static final String OVERVIEW_TRANS = "overview_trans";
  public static final String OVERVIEW_FAILURES = "overview_failures";
  public static final String STRUCT_FAILURE = "struct_failure";
  public static final String FAILURE = "failure";
  public static final String PACK = "pack";
  public static final String PACK_B = "pack_b";
  public static final String PACK_M = "pack_m";
  public static final String PACK_P = "pack_p";
  public static final String PACK_D = "pack_d";
  public static final String PACK_TRANS = "pack_trans";
  public static final String PACK_FAILURES = "pack_failures";
  public static final String SCEN = "scen";
  public static final String SCEN_C_TRANS = "scen_c_trans";
  public static final String SCEN_A_TRANS = "scen_a_trans";
  public static final String SCEN_FAILURES = "scen_failures";
  public static final String SPEC = "spec";
  public static final String SPEC_B = "spec_b";
  public static final String SPEC_M = "spec_m";
  public static final String SPEC_P = "spec_p";
  public static final String SPEC_D = "spec_d";
  public static final String SPEC_FAILURES = "spec_failures";
  public static final String METH = "meth";
  public static final String METH_FAILURES = "meth_failures";

  public String getTemplatePackage() {
    return TEMPLATE_PACKAGE;
  }

  public void copyAuxilaryFiles(File dir) throws Exception {
    String[] files = {
      "reportUTT.css",
      "logo.gif",
      "reportUTT.css",
      "bg.gif",
      "left.gif",
      "right.gif",
      "left_on.gif",
      "right_on.gif",
      "empty.gif",
      "generic.gif",
      "failure.gif",
      "package.gif",
      "scenario.gif",
      "specification.gif",
      "method.gif"
    };
    InputStream is;
    FileOutputStream os;
    for (int i = 0; i < files.length; i++) {
      String path = "/" + DocumentContext.GIF_PACKAGE + files[i];
      is = getClass().getResourceAsStream(path);
      System.out.println("file[" + i + "] = " + path + " is = " + is);
      os = new FileOutputStream(new File(dir, files[i]));
      copy(is, os);
      System.out.println("Copied to dir " + dir);
    }
  }

  protected static void copy(InputStream is, OutputStream os) {
    byte[] buffer = new byte[4096];
    for (; ; ) {
      int count = 0;
      System.out.println("===count = " + count);
      try {
        count = is.read(buffer);
        if (count < 0) {
          break;
        }
        os.write(buffer, 0, count);
      } catch (IOException ioe) {
        ioe.printStackTrace();
      }
    }
  }

  public String getRoot() {
    return "";
  }

  public String getTitle() {
    return "Generated ";
  }

  public String getTime() {
    return Calendar.getInstance().getTime().toString();
  }

  public ArrayList<String> getReportTypes() {
    ArrayList<String> reportTypes = new ArrayList<String>();
    reportTypes.add("summary");
    reportTypes.add("details");
    return reportTypes;
  }
}
