/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.test;

import com.unitesk.aspectrace.AspectAdapter;
import com.unitesk.aspectrace.TraceMessage;

import java.util.List;

/**
 * Adapter for aspect "test".
 *
 * @author Sergey Groshev
 */
public class TestAdapter extends AspectAdapter {

  @Override
  public String getAspect() {
    return "test";
  }

  @Override
  public Class<TestListener> getListenerClass() {
    return TestListener.class;
  }

  @Override
  protected void processMessage(TraceMessage msg) throws IllegalArgumentException {
    super.processMessage(msg);

    String msgName = msg.getName();
    if (msgName.equals("start")) {
      String name = msg.getAttr("name");
      for (TestListener lst : getListeners()) {
        lst.processTestStart(name);
      }
    } else if (msgName.equals("end")) {
      String name = msg.getAttr("name");
      boolean verdict = Boolean.parseBoolean(msg.getAttr("verdict"));
      for (TestListener lst : getListeners()) {
        lst.processTestEnd(name, verdict);
      }
    } else {
      throw new IllegalArgumentException("Unsupported test node: " + msgName);
    }
  }

  @Override
  protected List<TestListener> getListeners() {
    return (List<TestListener>) super.getListeners();
  }
}
