/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.tracer;

import com.unitesk.aspectrace.Tracer;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Properties;

/**
 * Writes trace to file.
 *
 * @author Sergey Groshev
 */
public class TraceFileWriter implements Tracer.Writer {

  FileOutputStream fos;
  Writer wrtr;

  public TraceFileWriter(File fl) throws IOException {
    File parent = fl.getParentFile();
    if (parent != null && !parent.exists()) {
      parent.mkdirs();
    }
    fos = new FileOutputStream(fl);
  }

  @Override
  public void startTrace(String prolog, Properties props)
      throws IOException, IllegalStateException {
    if (wrtr != null || fos == null) {
      throw new IllegalStateException("Tracing already started");
    }

    String encoding = props.getProperty(Tracer.PROP_FILE_ENCODING, Tracer.DEFAULT_ENCODING);
    wrtr = new BufferedWriter(new OutputStreamWriter(fos, encoding));
    fos = null; // Should not be reopened
    writeMessage(prolog);
  }

  @Override
  public void writeMessage(String msg) throws IOException, IllegalStateException {
    if (wrtr == null) {
      throw new IllegalStateException("Writing not started or already finished");
    }
    wrtr.write(msg);
  }

  @Override
  public void endTrace(String epilog) throws IOException, IllegalStateException {
    writeMessage(epilog);
    wrtr.close();
    wrtr = null;
  }
}
