/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.tracer;

import com.unitesk.aspectrace.AbstractTraceMessage;
import com.unitesk.aspectrace.MessageStream;
import com.unitesk.aspectrace.MessageStreamListener;
import com.unitesk.aspectrace.Tracer;

/**
 * Converts a trace to a PUSH-{@link MessageStream message stream}. Writes a trace which comes from
 * aspect tracers via {@link Tracer} directly to {@link
 * MessageStreamListener stream listners} without external representation.
 *
 * @author Sergey Groshev
 */
public class TraceToStream extends MessageStream implements Tracer.Channel {
  private boolean started = false;

  public TraceToStream() {}

  @Override
  public synchronized void startTrace(java.util.Properties props) {
    if (started) {
      return;
    }
    fireStreamStart();
    started = true;
  }

  @Override
  public synchronized void dispatch(AbstractTraceMessage msg)
      throws IllegalStateException, IllegalArgumentException {
    if (!started) {
      throw new IllegalStateException("Tracing has finished or not started yet");
    }
    fireStreamMessage(msg);
  }

  @Override
  public synchronized void endTrace() {
    if (started) { // May be called more than once
      fireStreamEnd(true);
    }

    started = false;
  }
}
